@@ -20,11 +20,6 @@
  *  Boston, MA 02110-1301 USA
  */
 
-/* GValueArray has deprecated without providing an alternative in glib >= 2.32
- * See https://bugzilla.gnome.org/show_bug.cgi?id=667228
- */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include "sysdeps.h"
 #include <math.h>
 #include <va/va.h>
@@ -36,27 +31,13 @@
 #include "gstvaapiencoder_h265.h"
 #include "gstvaapiutils_h265.h"
 #include "gstvaapiutils_h265_priv.h"
+#include "gstvaapiutils_h26x_priv.h"
 #include "gstvaapicodedbufferproxy_priv.h"
 #include "gstvaapisurface.h"
 
 #define DEBUG 1
 #include "gstvaapidebug.h"
 
-/* Define the maximum IDR period */
-#define MAX_IDR_PERIOD 512
-
-/* Default CPB length (in milliseconds) */
-#define DEFAULT_CPB_LENGTH 1500
-
-/* Scale factor for bitrate (HRD bit_rate_scale: min = 6) */
-#define SX_BITRATE 6
-
-/* Scale factor for CPB size (HRD cpb_size_scale: min = 4) */
-#define SX_CPB_SIZE 4
-
-/* Define default rate control mode ("constant-qp") */
-#define DEFAULT_RATECONTROL GST_VAAPI_RATECONTROL_CQP
-
 /* Supported set of VA rate controls, within this implementation */
 #define SUPPORTED_RATECONTROLS                          \
   (GST_VAAPI_RATECONTROL_MASK (CQP)) |                  \
@@ -205,66 +186,6 @@ h265_get_log2_max_pic_order_cnt (guint num)
   return ret;
 }
 
-/* ------------------------------------------------------------------------- */
-/* --- H.265 Bitstream Writer                                            --- */
-/* ------------------------------------------------------------------------- */
-
-#define WRITE_UINT32(bs, val, nbits) do {                       \
-    if (!gst_bit_writer_put_bits_uint32 (bs, val, nbits)) {     \
-      GST_WARNING ("failed to write uint32, nbits: %d", nbits); \
-      goto bs_error;                                            \
-    }                                                           \
-  } while (0)
-
-#define WRITE_UE(bs, val) do {                  \
-    if (!bs_write_ue (bs, val)) {               \
-      GST_WARNING ("failed to write ue(v)");    \
-      goto bs_error;                            \
-    }                                           \
-  } while (0)
-
-#define WRITE_SE(bs, val) do {                  \
-    if (!bs_write_se (bs, val)) {               \
-      GST_WARNING ("failed to write se(v)");    \
-      goto bs_error;                            \
-    }                                           \
-  } while (0)
-
-/* Write an unsigned integer Exp-Golomb-coded syntax element. i.e. ue(v) */
-static gboolean
-bs_write_ue (GstBitWriter * bs, guint32 value)
-{
-  guint32 size_in_bits = 0;
-  guint32 tmp_value = ++value;
-
-  while (tmp_value) {
-    ++size_in_bits;
-    tmp_value >>= 1;
-  }
-  if (size_in_bits > 1
-      && !gst_bit_writer_put_bits_uint32 (bs, 0, size_in_bits - 1))
-    return FALSE;
-  if (!gst_bit_writer_put_bits_uint32 (bs, value, size_in_bits))
-    return FALSE;
-  return TRUE;
-}
-
-/* Write a signed integer Exp-Golomb-coded syntax element. i.e. se(v) */
-static gboolean
-bs_write_se (GstBitWriter * bs, gint32 value)
-{
-  guint32 new_val;
-
-  if (value <= 0)
-    new_val = -(value << 1);
-  else
-    new_val = (value << 1) - 1;
-
-  if (!bs_write_ue (bs, new_val))
-    return FALSE;
-  return TRUE;
-}
-
 /* Write the NAL unit header */
 static gboolean
 bs_write_nal_header (GstBitWriter * bs, guint32 nal_unit_type)
@@ -1003,7 +924,7 @@ ensure_profile_limits (GstVaapiEncoderH265 * encoder)
       ("Needs to lower coding tools to meet target decoder constraints");
   GST_WARNING ("Only supporting Main profile, reset profile to Main");
 
-  encoder->profile = GST_VAAPI_PROFILE_H265_MAIN;;
+  encoder->profile = GST_VAAPI_PROFILE_H265_MAIN;
   encoder->profile_idc =
       gst_vaapi_utils_h265_get_profile_idc (encoder->profile);
 
@@ -1015,11 +936,16 @@ static gboolean
 ensure_profile (GstVaapiEncoderH265 * encoder)
 {
   GstVaapiProfile profile;
+  const GstVideoFormat format =
+      GST_VIDEO_INFO_FORMAT (GST_VAAPI_ENCODER_VIDEO_INFO (encoder));
 
   /* Always start from "Main" profile for maximum
      compatibility */
   profile = GST_VAAPI_PROFILE_H265_MAIN;
 
+  if (format == GST_VIDEO_FORMAT_P010_10LE)
+    profile = GST_VAAPI_PROFILE_H265_MAIN10;
+
   encoder->profile = profile;
   encoder->profile_idc = gst_vaapi_utils_h265_get_profile_idc (profile);
   return TRUE;
@@ -1510,6 +1436,13 @@ static gboolean
 fill_sequence (GstVaapiEncoderH265 * encoder, GstVaapiEncSequence * sequence)
 {
   VAEncSequenceParameterBufferHEVC *const seq_param = sequence->param;
+  const GstVideoFormat format =
+      GST_VIDEO_INFO_FORMAT (GST_VAAPI_ENCODER_VIDEO_INFO (encoder));
+  guint bits_depth_luma_minus8 =
+      GST_VIDEO_FORMAT_INFO_DEPTH (gst_video_format_get_info (format), 0);
+  if (bits_depth_luma_minus8 < 8)
+    return FALSE;
+  bits_depth_luma_minus8 -= 8;
 
   memset (seq_param, 0, sizeof (VAEncSequenceParameterBufferHEVC));
 
@@ -1531,8 +1464,8 @@ fill_sequence (GstVaapiEncoderH265 * encoder, GstVaapiEncSequence * sequence)
   seq_param->seq_fields.value = 0;
   seq_param->seq_fields.bits.chroma_format_idc = 1;
   seq_param->seq_fields.bits.separate_colour_plane_flag = 0;
-  seq_param->seq_fields.bits.bit_depth_luma_minus8 = 0;
-  seq_param->seq_fields.bits.bit_depth_chroma_minus8 = 0;
+  seq_param->seq_fields.bits.bit_depth_luma_minus8 = bits_depth_luma_minus8;
+  seq_param->seq_fields.bits.bit_depth_chroma_minus8 = bits_depth_luma_minus8;
   seq_param->seq_fields.bits.scaling_list_enabled_flag = FALSE;
   seq_param->seq_fields.bits.strong_intra_smoothing_enabled_flag = TRUE;
   seq_param->seq_fields.bits.amp_enabled_flag = TRUE;
@@ -1625,7 +1558,8 @@ fill_picture (GstVaapiEncoderH265 * encoder, GstVaapiEncPicture * picture,
   pic_param->num_ref_idx_l1_default_active_minus1 =
       (ref_pool->max_reflist1_count ? (ref_pool->max_reflist1_count - 1) : 0);
 
-  get_nal_unit_type (picture, &nal_unit_type);
+  if (!get_nal_unit_type (picture, &nal_unit_type))
+    return FALSE;
   pic_param->nal_unit_type = nal_unit_type;
 
   /* set picture fields */
@@ -2091,11 +2025,15 @@ gst_vaapi_encoder_h265_encode (GstVaapiEncoder * base_encoder,
     goto error;
 
   return GST_VAAPI_ENCODER_STATUS_SUCCESS;
+
+  /* ERRORS */
 error:
-  if (reconstruct)
-    gst_vaapi_encoder_release_surface (GST_VAAPI_ENCODER (encoder),
-        reconstruct);
-  return ret;
+  {
+    if (reconstruct)
+      gst_vaapi_encoder_release_surface (GST_VAAPI_ENCODER (encoder),
+          reconstruct);
+    return ret;
+  }
 }
 
 static GstVaapiEncoderStatus
@@ -2166,9 +2104,9 @@ gst_vaapi_encoder_h265_get_codec_data (GstVaapiEncoder * base_encoder,
   WRITE_UINT32 (&bs, 0x00, 2);  /* parallelismType */
   WRITE_UINT32 (&bs, 0x3f, 6);  /* 111111 */
   WRITE_UINT32 (&bs, 0x01, 2);  /* chroma_format_idc */
-  WRITE_UINT32 (&bs, 0x3f, 6);  /* 111111 */
+  WRITE_UINT32 (&bs, 0x1f, 5);  /* 11111 */
   WRITE_UINT32 (&bs, 0x01, 3);  /* bit_depth_luma_minus8 */
-  WRITE_UINT32 (&bs, 0x3f, 6);  /* 111111 */
+  WRITE_UINT32 (&bs, 0x1f, 5);  /* 11111 */
   WRITE_UINT32 (&bs, 0x01, 3);  /* bit_depth_chroma_minus8 */
   WRITE_UINT32 (&bs, 0x00, 16); /* avgFramerate */
   WRITE_UINT32 (&bs, 0x00, 2);  /* constatnFramerate */
@@ -2185,8 +2123,9 @@ gst_vaapi_encoder_h265_get_codec_data (GstVaapiEncoder * base_encoder,
   WRITE_UINT32 (&bs, GST_H265_NAL_VPS, 6);      /* Nal_unit_type */
   WRITE_UINT32 (&bs, 0x01, 16); /* numNalus, VPS count = 1 */
   g_assert (GST_BIT_WRITER_BIT_SIZE (&bs) % 8 == 0);
-  WRITE_UINT32 (&bs, vps_info.size, 16);        /* VPS nalUnitLength */
-  gst_bit_writer_put_bytes (&bs, vps_info.data, vps_info.size);
+  /* Write Nal unit length and data of VPS */
+  if (!gst_vaapi_utils_h26x_write_nal_unit (&bs, vps_info.data, vps_info.size))
+    goto nal_to_byte_stream_error;
 
   /* Write SPS */
   WRITE_UINT32 (&bs, 0x00, 1);  /* array_completeness */
@@ -2194,16 +2133,18 @@ gst_vaapi_encoder_h265_get_codec_data (GstVaapiEncoder * base_encoder,
   WRITE_UINT32 (&bs, GST_H265_NAL_SPS, 6);      /* Nal_unit_type */
   WRITE_UINT32 (&bs, 0x01, 16); /* numNalus, SPS count = 1 */
   g_assert (GST_BIT_WRITER_BIT_SIZE (&bs) % 8 == 0);
-  WRITE_UINT32 (&bs, sps_info.size, 16);        /* SPS nalUnitLength */
-  gst_bit_writer_put_bytes (&bs, sps_info.data, sps_info.size);
+  /* Write Nal unit length and data of SPS */
+  if (!gst_vaapi_utils_h26x_write_nal_unit (&bs, sps_info.data, sps_info.size))
+    goto nal_to_byte_stream_error;
 
   /* Write PPS */
   WRITE_UINT32 (&bs, 0x00, 1);  /* array_completeness */
   WRITE_UINT32 (&bs, 0x00, 1);  /* reserved zero */
   WRITE_UINT32 (&bs, GST_H265_NAL_PPS, 6);      /* Nal_unit_type */
   WRITE_UINT32 (&bs, 0x01, 16); /* numNalus, PPS count = 1 */
-  WRITE_UINT32 (&bs, pps_info.size, 16);        /* PPS nalUnitLength */
-  gst_bit_writer_put_bytes (&bs, pps_info.data, pps_info.size);
+  /* Write Nal unit length and data of PPS */
+  if (!gst_vaapi_utils_h26x_write_nal_unit (&bs, pps_info.data, pps_info.size))
+    goto nal_to_byte_stream_error;
 
   gst_buffer_unmap (encoder->pps_data, &pps_info);
   gst_buffer_unmap (encoder->sps_data, &sps_info);
@@ -2226,7 +2167,16 @@ bs_error:
     gst_buffer_unmap (encoder->sps_data, &sps_info);
     gst_buffer_unmap (encoder->pps_data, &pps_info);
     gst_bit_writer_clear (&bs, TRUE);
-    return FALSE;
+    return GST_VAAPI_ENCODER_STATUS_ERROR_OPERATION_FAILED;
+  }
+nal_to_byte_stream_error:
+  {
+    GST_ERROR ("failed to write nal unit");
+    gst_buffer_unmap (encoder->vps_data, &vps_info);
+    gst_buffer_unmap (encoder->sps_data, &sps_info);
+    gst_buffer_unmap (encoder->pps_data, &pps_info);
+    gst_bit_writer_clear (&bs, TRUE);
+    return GST_VAAPI_ENCODER_STATUS_ERROR_OPERATION_FAILED;
   }
 error_map_vps_buffer:
   {
