@@ -38,7 +38,11 @@
 #define GST_VAAPI_ENCODE_FLOW_CONVERT_ERROR     GST_FLOW_CUSTOM_ERROR_1
 
 GST_DEBUG_CATEGORY_STATIC (gst_vaapiencode_debug);
+#ifndef GST_DISABLE_GST_DEBUG
 #define GST_CAT_DEFAULT gst_vaapiencode_debug
+#else
+#define GST_CAT_DEFAULT NULL
+#endif
 
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (GstVaapiEncode,
     gst_vaapiencode, GST_TYPE_VIDEO_ENCODER,
@@ -63,11 +67,11 @@ static gboolean
 gst_vaapiencode_sink_query (GstVideoEncoder * encoder, GstQuery * query)
 {
   gboolean ret = TRUE;
-  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (encoder);
+  GstElement *const element = GST_ELEMENT (encoder);
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CONTEXT:
-      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      ret = gst_vaapi_handle_context_query (element, query);
       break;
     default:
       ret = GST_VIDEO_ENCODER_CLASS (gst_vaapiencode_parent_class)->sink_query
@@ -82,11 +86,11 @@ static gboolean
 gst_vaapiencode_src_query (GstVideoEncoder * encoder, GstQuery * query)
 {
   gboolean ret = TRUE;
-  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (encoder);
+  GstElement *const element = GST_ELEMENT (encoder);
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CONTEXT:
-      ret = gst_vaapi_handle_context_query (query, plugin->display);
+      ret = gst_vaapi_handle_context_query (element, query);
       break;
     default:
       ret = GST_VIDEO_ENCODER_CLASS (gst_vaapiencode_parent_class)->src_query
@@ -342,16 +346,87 @@ gst_vaapiencode_buffer_loop (GstVaapiEncode * encode)
   gst_pad_pause_task (GST_VAAPI_PLUGIN_BASE_SRC_PAD (encode));
 }
 
+static gboolean
+ensure_allowed_sinkpad_caps (GstVaapiEncode * encode)
+{
+  GstVaapiEncodeClass *klass = GST_VAAPIENCODE_GET_CLASS (encode);
+  GstCaps *out_caps, *raw_caps = NULL;
+  GArray *formats = NULL;
+  gboolean ret = FALSE;
+  GstVaapiProfile profile = GST_VAAPI_PROFILE_UNKNOWN;
+
+  if (encode->allowed_sinkpad_caps)
+    return TRUE;
+  if (!encode->encoder)
+    return TRUE;
+
+  out_caps = gst_caps_from_string (GST_VAAPI_MAKE_SURFACE_CAPS);
+  if (!out_caps)
+    goto failed_create_va_caps;
+
+  if (klass->get_profile) {
+    GstCaps *allowed =
+        gst_pad_get_allowed_caps (GST_VAAPI_PLUGIN_BASE_SRC_PAD (encode));
+    if (allowed) {
+      if (!gst_caps_is_empty (allowed) && !gst_caps_is_any (allowed))
+        profile = klass->get_profile (allowed);
+      gst_caps_unref (allowed);
+    }
+  }
+
+  formats = gst_vaapi_encoder_get_surface_formats (encode->encoder, profile);
+  if (!formats)
+    goto failed_get_formats;
+
+  raw_caps = gst_vaapi_video_format_new_template_caps_from_list (formats);
+  if (!raw_caps)
+    goto failed_create_raw_caps;
+
+  out_caps = gst_caps_make_writable (out_caps);
+  gst_caps_append (out_caps, gst_caps_copy (raw_caps));
+  gst_caps_replace (&encode->allowed_sinkpad_caps, out_caps);
+  GST_INFO_OBJECT (encode, "Allowed sink caps %" GST_PTR_FORMAT,
+      encode->allowed_sinkpad_caps);
+
+  ret = TRUE;
+
+bail:
+  if (out_caps)
+    gst_caps_unref (out_caps);
+  if (raw_caps)
+    gst_caps_unref (raw_caps);
+  if (formats)
+    g_array_unref (formats);
+  return ret;
+
+failed_create_va_caps:
+  {
+    GST_WARNING_OBJECT (encode, "failed to create VA/GL sink caps");
+    return FALSE;
+  }
+failed_get_formats:
+  {
+    GST_WARNING_OBJECT (encode, "failed to get allowed surface formats");
+    goto bail;
+  }
+failed_create_raw_caps:
+  {
+    GST_WARNING_OBJECT (encode, "failed to create raw sink caps");
+    goto bail;
+  }
+}
+
 static GstCaps *
 gst_vaapiencode_get_caps (GstVideoEncoder * venc, GstCaps * filter)
 {
-  GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (venc);
+  GstVaapiEncode *const encode = GST_VAAPIENCODE_CAST (venc);
   GstCaps *result;
 
-  result = gst_video_encoder_proxy_getcaps (venc, plugin->sinkpad_caps, filter);
-
-  GST_DEBUG_OBJECT (venc, "Returning sink caps %" GST_PTR_FORMAT, result);
+  ensure_allowed_sinkpad_caps (encode);
+  result = gst_video_encoder_proxy_getcaps (venc, encode->allowed_sinkpad_caps,
+      filter);
 
+  GST_DEBUG_OBJECT (venc, "Negotiated sink caps %" GST_PTR_FORMAT, result);
   return result;
 }
 
@@ -367,6 +442,8 @@ gst_vaapiencode_destroy (GstVaapiEncode * encode)
     gst_video_codec_state_unref (encode->output_state);
     encode->output_state = NULL;
   }
+
+  gst_caps_replace (&encode->allowed_sinkpad_caps, NULL);
   gst_vaapi_encoder_replace (&encode->encoder, NULL);
   return TRUE;
 }
@@ -438,13 +515,22 @@ gst_vaapiencode_open (GstVideoEncoder * venc)
   return success;
 }
 
+static gboolean
+gst_vaapiencode_start (GstVideoEncoder * venc)
+{
+  return ensure_encoder (GST_VAAPIENCODE_CAST (venc));
+}
+
+static gboolean
+gst_vaapiencode_stop (GstVideoEncoder * venc)
+{
+  return gst_vaapiencode_destroy (GST_VAAPIENCODE_CAST (venc));
+}
+
 static gboolean
 gst_vaapiencode_close (GstVideoEncoder * venc)
 {
-  GstVaapiEncode *const encode = GST_VAAPIENCODE_CAST (venc);
-
-  gst_vaapiencode_destroy (encode);
-  gst_vaapi_plugin_base_close (GST_VAAPI_PLUGIN_BASE (encode));
+  gst_vaapi_plugin_base_close (GST_VAAPI_PLUGIN_BASE (venc));
   return TRUE;
 }
 
@@ -470,11 +556,10 @@ static gboolean
 gst_vaapiencode_set_format (GstVideoEncoder * venc, GstVideoCodecState * state)
 {
   GstVaapiEncode *const encode = GST_VAAPIENCODE_CAST (venc);
+  gboolean ret;
 
   g_return_val_if_fail (state->caps != NULL, FALSE);
 
-  if (!ensure_encoder (encode))
-    return FALSE;
   if (!set_codec_state (encode, state))
     return FALSE;
 
@@ -487,8 +572,39 @@ gst_vaapiencode_set_format (GstVideoEncoder * venc, GstVideoCodecState * state)
   encode->input_state = gst_video_codec_state_ref (state);
   encode->input_state_changed = TRUE;
 
-  return gst_pad_start_task (GST_VAAPI_PLUGIN_BASE_SRC_PAD (encode),
+  ret = gst_pad_start_task (GST_VAAPI_PLUGIN_BASE_SRC_PAD (encode),
       (GstTaskFunction) gst_vaapiencode_buffer_loop, encode, NULL);
+
+  if (!ret)
+    return FALSE;
+
+  /* Store some tags */
+  {
+    GstTagList *tags = gst_tag_list_new_empty ();
+    const gchar *encoder, *codec;
+    guint bitrate = 0;
+
+    g_object_get (encode, "bitrate", &bitrate, NULL);
+    gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_NOMINAL_BITRATE,
+        bitrate, NULL);
+
+    if ((encoder =
+            gst_element_class_get_metadata (GST_ELEMENT_GET_CLASS (encode),
+                GST_ELEMENT_METADATA_LONGNAME)))
+      gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_ENCODER, encoder,
+          NULL);
+
+    if ((codec =
+            gst_vaapi_codec_get_name (gst_vaapi_profile_get_codec
+                (gst_vaapi_profile_from_caps (state->caps)))))
+      gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_CODEC, codec,
+          NULL);
+
+    gst_video_encoder_merge_tags (venc, tags, GST_TAG_MERGE_REPLACE);
+    gst_tag_list_unref (tags);
+  }
+
+  return TRUE;
 }
 
 static GstFlowReturn
@@ -713,6 +829,8 @@ gst_vaapiencode_class_init (GstVaapiEncodeClass * klass)
 
   venc_class->open = GST_DEBUG_FUNCPTR (gst_vaapiencode_open);
   venc_class->close = GST_DEBUG_FUNCPTR (gst_vaapiencode_close);
+  venc_class->start = GST_DEBUG_FUNCPTR (gst_vaapiencode_start);
+  venc_class->stop = GST_DEBUG_FUNCPTR (gst_vaapiencode_stop);
   venc_class->set_format = GST_DEBUG_FUNCPTR (gst_vaapiencode_set_format);
   venc_class->handle_frame = GST_DEBUG_FUNCPTR (gst_vaapiencode_handle_frame);
   venc_class->finish = GST_DEBUG_FUNCPTR (gst_vaapiencode_finish);
